@@ -22,11 +22,20 @@ from trace_slideshow import (
     TrainHTMLSlideShow, SectExecuteHTMLSlideShow, ExecuteHTMLSlideShow
 )
 from facade import (
-    INSPECTION_STYLE, SLIDE_CSS_FILENAME, inspect_training,
-    show_training, inspect_execution, show_execution
+    INSPECTION_STYLE, SLIDE_CSS_FILENAME, EmptyTraceException,
+    inspect_training, show_training, inspect_execution, show_execution
 )
 
 del bihinet_translator
 del trace_inspection
 del trace_slideshow
-del facade
\ No newline at end of file
+del facade
+
+from mdp.utils import fixup_namespace
+fixup_namespace(__name__, None,
+                ('bihinet_translator',
+                 'trace_inspection',
+                 'trace_slideshow',
+                 'facade',
+                 ))
+del fixup_namespace
