@@ -56,7 +56,7 @@ class CovarianceMatrix(object):
 
     def _init_internals(self, x):
         """Init the internal structures.
-        
+
         The reason this is not done in the constructor is that we want to be
         able to derive the input dimension and the dtype directly from the
         data this class receives.
@@ -74,7 +74,7 @@ class CovarianceMatrix(object):
 
     def update(self, x):
         """Update internal structures.
-        
+
         Note that no consistency checks are performed on the data (this is
         typically done in the enclosing node).
         """
@@ -88,10 +88,13 @@ class CovarianceMatrix(object):
         self._avg += x.sum(axis=0)
         self._tlen += x.shape[0]
 
-    def fix(self):
+    def fix(self, center=True):
         """Returns a triple containing the covariance matrix, the average and
         the number of observations. The covariance matrix is then reset to
-        a zero-state."""
+        a zero-state.
+
+        If center is false, the returned matrix is the matrix of the second moments,
+        i.e. the covariance matrix of the data without subtracting the mean."""
         # local variables
         type_ = self._dtype
         tlen = self._tlen
@@ -101,15 +104,19 @@ class CovarianceMatrix(object):
 
         ##### fix the training variables
         # fix the covariance matrix (try to do everything inplace)
-        avg_mtx = numx.outer(avg, avg)
-
         if self.bias:
-            avg_mtx /= tlen*(tlen)
             cov_mtx /= tlen
         else:
-            avg_mtx /= tlen*(tlen - 1)
             cov_mtx /= tlen - 1
-        cov_mtx -= avg_mtx
+
+        if center:
+            avg_mtx = numx.outer(avg, avg)
+            if self.bias:
+                avg_mtx /= tlen*(tlen)
+            else:
+                avg_mtx /= tlen*(tlen - 1)
+            cov_mtx -= avg_mtx
+
         # fix the average
         avg /= tlen
 
@@ -124,7 +131,7 @@ class CovarianceMatrix(object):
         return cov_mtx, avg, tlen
 
 
-class DelayCovarianceMatrix(object):    
+class DelayCovarianceMatrix(object):
     """This class stores an empirical covariance matrix between the signal and
     time delayed signal that can be updated incrementally.
 
@@ -169,7 +176,7 @@ class DelayCovarianceMatrix(object):
         the constructor is that we want to be able to derive the input
         dimension and the dtype directly from the data this class receives.
         """
-        
+
         # init dtype
         if self._dtype is None:
             self._dtype = x.dtype
@@ -196,7 +203,7 @@ class DelayCovarianceMatrix(object):
         if tlen < (dt+1):
             err = 'Block length is %d, should be at least %d.' % (tlen, dt+1)
             raise mdp.MDPException(err)
-        
+
         # update the covariance matrix, the average and the number of
         # observations (try to do everything inplace)
         self._cov_mtx += mdp.utils.mult(x[:tlen-dt, :].T, x[dt:tlen, :])
@@ -213,11 +220,11 @@ class DelayCovarianceMatrix(object):
         the average <x(t)> over the first N-dt points, the average of the
         delayed signal <x(t+dt)> and the number of observations. The internal
         data is then reset to a zero-state.
-        
+
         If A is defined, the covariance matrix is transformed by the linear
         transformation Ax . E.g. to whiten the data, A is the whitening matrix.
         """
-        
+
         # local variables
         type_ = self._dtype
         tlen = self._tlen
@@ -230,7 +237,7 @@ class DelayCovarianceMatrix(object):
         # fix the covariance matrix (try to do everything inplace)
         avg_mtx = numx.outer(avg, avg_dt)
         avg_mtx /= tlen
-                 
+
         cov_mtx -= avg_mtx
         if self.bias:
             cov_mtx /= tlen
@@ -239,7 +246,7 @@ class DelayCovarianceMatrix(object):
 
         if A is not None:
             cov_mtx = mdp.utils.mult(A, mdp.utils.mult(cov_mtx, A.T))
-        
+
         # fix the average
         avg /= tlen
         avg_dt /= tlen
@@ -312,7 +319,7 @@ class MultipleCovarianceMatrices(object):
         covs[i, :, :], covs[j, :, :] = covs[j, :, :], covs[i, :, :] + 0
         covs[:, i, :], covs[:, j, :] = covs[:, j, :], covs[:, i, :] + 0
         self.covs = covs
-        
+
     def transform(self, trans_matrix):
         """Apply a linear transformation to all matrices, defined by the
         transformation matrix."""
@@ -321,7 +328,7 @@ class MultipleCovarianceMatrices(object):
             self.covs[:, :, cov] = mdp.utils.mult(
                 mdp.utils.mult(trans_matrix.T, self.covs[:, :, cov]),
                 trans_matrix)
-    
+
     def copy(self):
         """Return a deep copy of the instance."""
         return MultipleCovarianceMatrices(self.covs.transpose([2, 0, 1]))
@@ -341,7 +348,7 @@ class CrossCovarianceMatrix(CovarianceMatrix):
         type_ = self._dtype
         self._cov_mtx = numx.zeros((dim_x, dim_y), type_)
         self._avgx = numx.zeros(dim_x, type_)
-        self._avgy = numx.zeros(dim_y, type_) 
+        self._avgy = numx.zeros(dim_y, type_)
 
 
     def update(self, x, y):
@@ -350,7 +357,7 @@ class CrossCovarianceMatrix(CovarianceMatrix):
             err = '# samples mismatch: x (%d) != y (%d)'%(x.shape[0],
                                                           y.shape[0])
             raise mdp.MDPException(err)
-        
+
         if self._cov_mtx is None:
             self._init_internals(x, y)
 
@@ -396,4 +403,3 @@ class CrossCovarianceMatrix(CovarianceMatrix):
         self._tlen = 0
 
         return cov_mtx, avgx, avgy, tlen
-        
